import pygame
from unittest import mock
from pytest_devlife import util
from pytest_devlife.fixtures import mockgame
import inspect
try:
    import jogo
except ImportError:
    jogo = None


def test_ponto_fora_circulo_na_origem():
    util.function_exists_in_module(jogo, 'colisao_ponto_circulo')

    res = jogo.colisao_ponto_circulo(1, 1, 0, 0, 1)
    assert res == False, f'Círculo (raio=1): (0, 0), Ponto (1, 1)'


def test_ponto_fora_circulo():
    util.function_exists_in_module(jogo, 'colisao_ponto_circulo')

    res = jogo.colisao_ponto_circulo(1, -1, 10, 10, 2)
    assert res == False, f'Círculo (raio=2): (10, 10), Ponto (1, -1)'


def test_ponto_fora_circulo_negativo():
    util.function_exists_in_module(jogo, 'colisao_ponto_circulo')

    res = jogo.colisao_ponto_circulo(-10.01, 2, -10, 0, 2)
    assert res == False, f'Círculo (raio=2): (-10, 0), Ponto (-10, 2)'


def test_ponto_dentro_circulo_na_origem():
    util.function_exists_in_module(jogo, 'colisao_ponto_circulo')

    res = jogo.colisao_ponto_circulo(-1, -1, 0, 0, 2)
    assert res == True, f'Círculo (raio=2): (0, 0), Ponto (-1, -1)'


def test_ponto_dentro_circulo_na_borda():
    util.function_exists_in_module(jogo, 'colisao_ponto_circulo')

    res = jogo.colisao_ponto_circulo(-1, 0, 0, 0, 1)
    assert res == True, f'Círculo (raio=1): (0, 0), Ponto (-1, 0)'


def test_ponto_dentro_circulo():
    util.function_exists_in_module(jogo, 'colisao_ponto_circulo')

    res = jogo.colisao_ponto_circulo(8, 3, 8, 3, 0.5)
    assert res == True, f'Círculo (raio=0.5): (8, 3), Ponto (8, 3)'


def test_ponto_fora_retangulo():
    util.function_exists_in_module(jogo, 'colisao_ponto_retangulo')
    rect = (200, 300, 40, 30)
    pontos = [
        (100, 200),
        (220, 150),
        (250, 280),
        (90, 315),
        (250, 310),
        (50, 340),
        (230, 334),
        (270, 331)
    ]
    for p in pontos:
        res = jogo.colisao_ponto_retangulo(*p, *rect)
        assert res == False, f'Rect ({rect}), Ponto ({p})'


def test_ponto_dentro_retangulo():
    util.function_exists_in_module(jogo, 'colisao_ponto_retangulo')
    rect = (0, 300, 40, 30)
    
    res = jogo.colisao_ponto_retangulo(20, 301, *rect)
    assert res == True, f'Rect ({rect}), Ponto ({(20, 301)})'
